﻿using RenovationWorkContracts.BindingModels;
using RenovationWorkContracts.ViewModels;
using RenovationWorkDataModels.Enums;
using RenovationWorkDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace RenovationWorkDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        [Required]
        public int ClientId { get; private set; }
        public virtual Client Client { get; private set; }
        [Required]
        public int RepairId { get; private set; }
        public virtual Repair Repair { get; set; } = new();
        [Required]
        public int Count { get; private set; }
        [Required]
        public double Sum { get; private set; }
        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        public DateTime? DateImplement { get; private set; }
        public int? ImplementerId { get; private set; } = null;
        public virtual Implementer? Implementer { get; private set; }

        public static Order Create(RenovationWorkDatabase context, OrderBindingModel model)
        {
            return new Order()
            {
                Id = model.Id,
                RepairId = model.RepairId,
                Repair = context.Repairs.First(x => x.Id == model.RepairId),
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            RepairId =  RepairId,
            RepairName = Repair.RepairName,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            ClientId = ClientId,
            ClientFIO = Client?.ClientFIO ?? string.Empty,
            ImplementerId = ImplementerId,
            ImplementerFIO = Implementer != null ? Implementer.ImplementerFIO : string.Empty
        };
    }
}