﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using RenovationWorkContracts.BindingModels;
using RenovationWorkContracts.ViewModels;
using RenovationWorkDataModels.Enums;
using RenovationWorkDataModels.Models;

namespace RenovationWorkListImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        public int RepairId { get; private set; }
        public int ClientId { get; private set; }
        public int? ImplementerId { get; private set; }
        public int Count { get; private set; }
        public double Sum { get; private set; }

        public OrderStatus Status { get; set; } = OrderStatus.Неизвестен;

        public DateTime DateCreate { get; set; } = DateTime.Now;

        public DateTime? DateImplement { get; set; }

        public static Order? Create(OrderBindingModel? model)
        {
            if (model == null)
            {
                return null;
            }
            return new Order()
            {
                Id = model.Id,
                RepairId = model.RepairId,
                ClientId = model.ClientId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                ImplementerId = model.ImplementerId
            };
        }
        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Status = model.Status;
            if (model.DateImplement != null)
            {
                DateImplement = model.DateImplement;
                ImplementerId = model.ImplementerId;
            }
        }
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            RepairId = RepairId,
            ClientId = ClientId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            ImplementerId = ImplementerId,
        };
    }
}

