﻿using Microsoft.AspNetCore.Mvc;
using RenovationWorkContracts.BindingModels;
using RenovationWorkContracts.BusinessLogicsContracts;
using RenovationWorkContracts.SearchModels;
using RenovationWorkContracts.ViewModels;

namespace RenovationWorkRestApi.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class MainController : Controller
    {
        private readonly ILogger _logger;
        private readonly IOrderLogic _order;
        private readonly IRepairLogic _repair;
        public MainController(ILogger<MainController> logger, IOrderLogic order, IRepairLogic repair)
        {
            _logger = logger;
            _order = order;
            _repair = repair;
        }
        [HttpGet]
        public List<RepairViewModel>? GetRepairList()
        {
            try
            {
                return _repair.ReadList(null);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения списка кузнечных изделий");
                throw;
            }
        }
        [HttpGet]
        public RepairViewModel? GetRepair(int repairId)
        {
            try
            {
                return _repair.ReadElement(new RepairSearchModel
                {
                    Id = repairId
                });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения ремонтного изделия по id={Id}", repairId);
                throw;
            }
        }
        [HttpGet]
        public List<OrderViewModel>? GetOrders(int clientId)
        {
            try
            {
                return _order.ReadList(new OrderSearchModel
                {
                    ClientId = clientId
                });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения списка заказов клиента id ={ Id}", clientId);
                throw;
            }
        }
        [HttpPost]
        public void CreateOrder(OrderBindingModel model)
        {
            try
            {
                _order.CreateOrder(model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания заказа");
                throw;
            }
        }
    }
}